@@ -33,14 +33,15 @@
 
 Imagecodecs is a Python library that provides block-oriented, in-memory buffer
 transformation, compression, and decompression functions for use in the
-tifffile, czifile, and other scientific image input/output modules.
+tifffile, czifile, zarr, and other scientific image input/output modules.
 
 Decode and/or encode functions are implemented for Zlib (DEFLATE), GZIP,
 ZStandard (ZSTD), Blosc, Brotli, Snappy, LZMA, BZ2, LZ4, LZ4F, LZ4HC,
-LZW, LZF, ZFP, AEC, LERC, NPY, PNG, GIF, TIFF, WebP, JPEG 8-bit, JPEG 12-bit,
-Lossless JPEG (LJPEG, SOF3), JPEG 2000, JPEG LS, JPEG XR, JPEG XL, AVIF,
-PackBits, Packed Integers, Delta, XOR Delta, Floating Point Predictor,
-Bitorder reversal, Bitshuffle, and Float24 (24-bit floating point).
+LZW, LZF, PGLZ (PostgreSQL LZ), ZFP, AEC, LERC, NPY, PNG, GIF, TIFF, WebP,
+JPEG 8-bit, JPEG 12-bit, Lossless JPEG (LJPEG, SOF3), JPEG 2000, JPEG LS,
+JPEG XR (WDP, HD Photo), JPEG XL, AVIF, PackBits, Packed Integers, Delta,
+XOR Delta, Floating Point Predictor, Bitorder reversal, Bitshuffle, and
+Float24 (24-bit floating point).
 
 :Author:
   `Christoph Gohlke <https://www.lfd.uci.edu/~gohlke/>`_
@@ -50,7 +51,7 @@ Bitorder reversal, Bitshuffle, and Float24 (24-bit floating point).
 
 :License: BSD 3-Clause
 
-:Version: 2021.2.26
+:Version: 2021.4.28
 
 :Status: Alpha
 
@@ -59,25 +60,26 @@ Requirements
 This release has been tested with the following requirements and dependencies
 (other versions may work):
 
-* `CPython 3.7.9, 3.8.8, 3.9.2 64-bit <https://www.python.org>`_
+* `CPython 3.7.9, 3.8.9, 3.9.4 64-bit <https://www.python.org>`_
 * `Numpy 1.19.5 <https://pypi.org/project/numpy/>`_
-* `Cython 0.29.22 <https://cython.org>`_
+* `Cython 0.29.23 <https://cython.org>`_
 * `zlib 1.2.11 <https://github.com/madler/zlib>`_
 * `lz4 1.9.3 <https://github.com/lz4/lz4>`_
-* `zstd 1.4.8 <https://github.com/facebook/zstd>`_
+* `zstd 1.4.9 <https://github.com/facebook/zstd>`_
 * `blosc 1.21.0 <https://github.com/Blosc/c-blosc>`_
 * `bzip2 1.0.8 <https://sourceware.org/bzip2>`_
 * `liblzma 5.2.5 <https://github.com/xz-mirror/xz>`_
 * `liblzf 3.6 <http://oldhome.schmorp.de/marc/liblzf.html>`_
 * `libpng 1.6.37 <https://github.com/glennrp/libpng>`_
 * `libwebp 1.2.0 <https://github.com/webmproject/libwebp>`_
-* `libtiff 4.2.0 <https://gitlab.com/libtiff/libtiff>`_
-* `libjpeg-turbo 2.0.6 <https://github.com/libjpeg-turbo/libjpeg-turbo>`_
+* `libtiff 4.3.0 <https://gitlab.com/libtiff/libtiff>`_
+* `libjpeg-turbo 2.1.0 <https://github.com/libjpeg-turbo/libjpeg-turbo>`_
   (8 and 12-bit)
 * `libjpeg 9d <http://libjpeg.sourceforge.net/>`_
 * `charls 2.2.0 <https://github.com/team-charls/charls>`_
 * `openjpeg 2.4.0 <https://github.com/uclouvain/openjpeg>`_
 * `jxrlib 1.1 <https://packages.debian.org/source/sid/jxrlib>`_
+* `jpeg-xl 0.3.7 <https://gitlab.com/wg1/jpeg-xl>`_
 * `zfp 0.5.5 <https://github.com/LLNL/zfp>`_
 * `bitshuffle 0.3.5 <https://github.com/kiyo-masui/bitshuffle>`_
 * `libaec 1.0.4 <https://gitlab.dkrz.de/k202009/libaec>`_
@@ -90,22 +92,24 @@ This release has been tested with the following requirements and dependencies
 * `libdeflate 1.7 <https://github.com/ebiggers/libdeflate>`_
 * `libavif 0.9.0 <https://github.com/AOMediaCodec/libavif>`_
 * `dav1d 0.8.2 <https://github.com/videolan/dav1d>`_
-* `rav1e 0.4.0 <https://github.com/xiph/rav1e>`_
+* `rav1e 0.4.1 <https://github.com/xiph/rav1e>`_
 * `aom 2.0.2 <https://aomedia.googlesource.com/aom>`_
 * `lcms 2.12 <https://github.com/mm2/Little-CMS>`_
 
 Required Python packages for testing (other versions may work):
 
-* `tifffile 2021.2.26 <https://pypi.org/project/tifffile/>`_
+* `tifffile 2021.4.8 <https://pypi.org/project/tifffile/>`_
 * `czifile 2019.7.2 <https://pypi.org/project/czifile/>`_
 * `python-blosc 1.10.2 <https://github.com/Blosc/python-blosc>`_
 * `python-lz4 3.1.3 <https://github.com/python-lz4/python-lz4>`_
-* `python-zstd 1.4.8.1 <https://github.com/sergey-dryabzhinsky/python-zstd>`_
+* `python-zstd 1.4.9.1 <https://github.com/sergey-dryabzhinsky/python-zstd>`_
 * `python-lzf 0.2.4 <https://github.com/teepark/python-lzf>`_
 * `python-brotli 1.0.9 <https://github.com/google/brotli/tree/master/python>`_
 * `python-snappy 0.6.0 <https://github.com/andrix/python-snappy>`_
 * `zopflipy 1.5 <https://github.com/hattya/zopflipy>`_
 * `bitshuffle 0.3.5 <https://github.com/kiyo-masui/bitshuffle>`_
+* `numcodecs 0.7.3 <https://github.com/zarr-developers/numcodecs>`_
+* `zarr 2.8.1 <https://github.com/zarr-developers/zarr-python>`_
 
 Notes
 -----
@@ -126,9 +130,13 @@ Refer to the imagecodecs/licenses folder for 3rd-party library licenses.
 
 This software is based in part on the work of the Independent JPEG Group.
 
-This software includes modified versions of `dcm2niix's jpg_0XC3.cpp
+This software includes a modified version of `dcm2niix's jpg_0XC3.cpp
 <https://github.com/rordenlab/dcm2niix/blob/master/console/jpg_0XC3.cpp>`_.
 
+This software includes a modified version of `PostgreSQL's pg_lzcompress.c
+<https://github.com/postgres/postgres/blob/REL_13_STABLE/src/common/
+pg_lzcompress.c>`_.
+
 This software includes a copy of `liblj92
 <https://bitbucket.org/baldand/mlrawviewer/src/master/liblj92/>`_.
 
@@ -147,7 +155,7 @@ Install the requirements for building imagecodecs from source code on
 latest Ubuntu Linux distributions:
 
     ``sudo apt-get install build-essential python3-dev cython3
-    python3-setuptools python3-pip python3-wheel python3-numpy
+    python3-setuptools python3-pip python3-wheel python3-numpy python3-zarr
     python3-pytest python3-blosc python3-brotli python3-snappy python3-lz4
     libz-dev libblosc-dev liblzma-dev liblz4-dev libzstd-dev libpng-dev
     libwebp-dev libbz2-dev libopenjp2-7-dev libjpeg-dev libjxr-dev
@@ -185,7 +193,6 @@ Other Python packages and C libraries providing imaging or compression codecs:
 * `OpenEXR <https://github.com/AcademySoftwareFoundation/openexr>`_
 * `tinyexr <https://github.com/syoyo/tinyexr>`_
 * `pytinyexr <https://github.com/syoyo/pytinyexr>`_
-* `jpeg-xl <https://gitlab.com/wg1/jpeg-xl>`_
 * `libjpeg <https://github.com/thorfdbg/libjpeg>`_ (GPL)
 * `pylibjpeg <https://github.com/pydicom/pylibjpeg>`_
 * `pylibjpeg-libjpeg <https://github.com/pydicom/pylibjpeg-libjpeg>`_ (GPL)
@@ -196,8 +203,25 @@ Other Python packages and C libraries providing imaging or compression codecs:
 
 Revisions
 ---------
+2021.4.28
+    Pass 5119 tests.
+    Change WebP default compression level to lossless.
+    Rename jpegxl codec to brunsli (breaking).
+    Add new JPEG XL codec via jpeg-xl library.
+    Add PGLZ codec via PostgreSQL's pg_lzcompress.c.
+    Update to libtiff 4.3 and libjpeg-turbo 2.1.
+    Enable JPEG 12-bit codec in manylinux wheels.
+    Drop manylinux2010 wheels.
+2021.3.31
+    Add numcodecs compatible codecs for use by Zarr (experimental).
+    Support separate JPEG header in jpeg_decode.
+    Do not decode JPEG LS and XL in jpeg_decode (breaking).
+    Fix ZFP with partial header.
+    Fix JPEG LS tests (#15).
+    Fix LZ4F contentchecksum.
+    Remove blosc Snappy tests.
+    Fix docstrings.
 2021.2.26
-    Pass 4915 tests.
     Support X2 and X4 floating point predictors (found in DNG).
 2021.1.28
     Add option to return JPEG XR fixed point pixel types as integers.
@@ -294,7 +318,7 @@ Refer to the CHANGES file for older revisions.
 
 """
 
-__version__ = '2021.2.26'
+__version__ = '2021.4.28'
 
 import os
 import sys
@@ -335,6 +359,7 @@ _API = {
     'bitshuffle': [],
     'blosc': [],
     'brotli': [],
+    'brunsli': [],
     'bz2': [],
     'deflate': ['deflate_crc32', 'deflate_adler32', ('deflate', 'gzip')],
     'gif': [],
@@ -351,6 +376,7 @@ _API = {
     'lz4f': [],
     'lzf': [],
     'lzma': [],
+    'pglz': [],
     'png': [],
     'snappy': [],
     # 'szip': [],
@@ -425,12 +451,14 @@ def _load_all():
 
 
 def __dir__():
-    """Module __dir__."""
+    """Return list of attribute names accessible on module."""
     return sorted(list(_ATTRIBUTES) + list(_COMPATIBILITY))
 
 
 def __getattr__(name):
-    """Load attribute's extension and add its attributes to package namespace.
+    """Return module attribute after loading it from extension.
+
+    Load attribute's extension and add its attributes to the package namespace.
 
     """
     name_ = name
@@ -471,48 +499,43 @@ def __getattr__(name):
 
 
 class DelayedImportError(ImportError):
+    """Delayed ImportError."""
+
     def __init__(self, name):
+        """Initialize instance from attribute name."""
         msg = f"could not import name {name!r} from 'imagecodecs'"
         super().__init__(msg)
 
 
 def _stub(name, module):
     """Return stub function or class."""
-
     if name.endswith('_version'):
         if module is None:
 
             def stub_version():
-                f"""Stub for imagecodecs.{name}."""
+                """Stub for imagecodecs.codec_version function."""
                 return f"{name[:-8]} n/a"
 
         else:
 
             def stub_version():
-                f"""Stub for imagecodecs.{name}."""
+                """Stub for imagecodecs.codec_version function."""
                 return f"{name[:-8]} unknow"
 
         return stub_version
 
     if name.endswith('_check'):
-        if module is None:
 
-            def stub_check(arg):
-                f"""Stub for imagecodecs.{name}."""
-                return False
-
-        else:
-
-            def stub_check(arg):
-                f"""Stub for imagecodecs.{name}."""
-                return None
+        def stub_check(arg):
+            """Stub for imagecodecs.codec_check function."""
+            return False
 
         return stub_check
 
     if name.endswith('_decode'):
 
         def stub_decode(*args, **kwargs):
-            f"""Stub for imagecodecs.{name}."""
+            """Stub for imagecodecs.codec_decode function."""
             raise DelayedImportError(name)
 
         return stub_decode
@@ -520,7 +543,7 @@ def _stub(name, module):
     if name.endswith('_encode'):
 
         def stub_encode(*args, **kwargs):
-            f"""Stub for imagecodecs.{name}."""
+            """Stub for imagecodecs.codec_encode function."""
             raise DelayedImportError(name)
 
         return stub_encode
@@ -528,7 +551,7 @@ def _stub(name, module):
     if name.islower():
 
         def stub_function(*args, **kwargs):
-            f"""Stub for imagecodecs.{name}."""
+            """Stub for imagecodecs.codec_function."""
             raise DelayedImportError(name)
 
         return stub_function
@@ -536,7 +559,7 @@ def _stub(name, module):
     if name.endswith('Error'):
 
         class StubError(RuntimeError):
-            f"""Stub for imagecodecs.{name}."""
+            """Stub for imagecodecs.CodecError class."""
 
             def __init__(self, *args, **kwargs):
                 raise DelayedImportError(name)
@@ -555,12 +578,12 @@ def _stub(name, module):
     if name.isupper():
 
         class STUB(metaclass=StubType):
-            f"""Stub for imagecodecs.{name}."""
+            """Stub for imagecodecs.CODEC constants."""
 
         return STUB
 
     class Stub(metaclass=StubType):
-        f"""Stub for imagecodecs.{name}."""
+        """Stub for imagecodecs.Codec class."""
 
     return Stub
 
@@ -621,7 +644,7 @@ def imread(fileobj, codec=None, memmap=True, return_codec=False, **kwargs):
         if ext in _imcodecs():
             codec = _imcodecs()[ext]
             if codec == 'jpeg':
-                codecs.extend(('jpeg8', 'jpeg12', 'jpegls', 'jpegsof3'))
+                codecs.extend(('jpeg8', 'jpeg12', 'jpegsof3', 'ljpeg'))
             else:
                 codecs.append(codec)
         # try other imaging codecs
@@ -640,6 +663,8 @@ def imread(fileobj, codec=None, memmap=True, return_codec=False, **kwargs):
                 'jpegxr',
                 'jpegxl',
                 'avif',
+                'ljpeg',
+                # 'brunsli',
                 # 'exr',
                 'zfp',
                 'lerc',
@@ -756,12 +781,13 @@ def _imcodecs(_codecs_={}):
     if not _codecs_:
         codecs = {
             'avif': ('avif', 'avifs'),
+            'brunsli': ('brn',),
             # 'exr': ('exr',),
             'gif': ('gif',),
             'jpeg': ('jpg', 'jpeg', 'jpe', 'jfif', 'jif', 'ljpeg'),
             'jpeg2k': ('j2k', 'jp2', 'j2c', 'jpc', 'jpx', 'jpf'),  # jpm, mj2
             'jpegls': ('jls',),
-            'jpegxl': ('jxl', 'brn'),
+            'jpegxl': ('jxl',),
             'jpegxr': ('jxr', 'hdp', 'wdp'),
             'lerc': ('lerc1', 'lerc2'),
             'numpy': ('npy', 'npz'),
@@ -856,14 +882,15 @@ def jpeg_decode(
     data,
     bitspersample=None,
     tables=None,
+    header=None,
     colorspace=None,
     outcolorspace=None,
     shape=None,
     out=None,
 ):
-    """Decode JPEG 8-bit, 12-bit, SOF3, LS, or XL.
-
-    """
+    """Decode JPEG 8-bit, 12-bit, and SOF3."""
+    if header is not None:
+        data = header + data + b'\xff\xd9'
     if bitspersample is None:
         try:
             return imagecodecs.jpeg8_decode(
@@ -876,9 +903,7 @@ def jpeg_decode(
             )
         except Exception as exc:
             msg = str(exc)
-            if 'Empty JPEG image' in msg:
-                # TODO: handle Hamamatsu NDPI slides with dimensions > 65500
-                raise exc
+
             if 'Unsupported JPEG data precision' in msg:
                 return imagecodecs.jpeg12_decode(
                     data,
@@ -889,15 +914,14 @@ def jpeg_decode(
                     out=out,
                 )
             if 'SOF type' in msg:
-                return imagecodecs.jpegsof3_decode(data, out=out)
-            # Unsupported marker type
-            try:
-                return imagecodecs.jpegls_decode(data, out=out)
-            except Exception:
                 try:
-                    return imagecodecs.jpegxl_decode(data, out=out)
+                    return imagecodecs.jpegsof3_decode(data, out=out)
                 except Exception:
-                    raise exc
+                    return imagecodecs.ljpeg_decode(data, out=out)
+            # if 'Empty JPEG image' in msg:
+            # e.g. Hamamatsu NDPI slides with dimensions > 65500
+            # Unsupported marker type
+            raise exc
     try:
         if bitspersample == 8:
             return imagecodecs.jpeg8_decode(
@@ -917,23 +941,19 @@ def jpeg_decode(
                 shape=shape,
                 out=out,
             )
-        try:
-            return imagecodecs.jpegsof3_decode(data, out=out)
-        except Exception:
-            return imagecodecs.jpegls_decode(data, out=out)
+            try:
+                return imagecodecs.jpegsof3_decode(data, out=out)
+            except Exception:
+                return imagecodecs.ljpeg_decode(data, out=out)
     except Exception as exc:
         msg = str(exc)
-        if 'Empty JPEG image' in msg:
-            raise exc
         if 'SOF type' in msg:
-            return imagecodecs.jpegsof3_decode(data, out=out)
-        try:
-            return imagecodecs.jpegls_decode(data, out=out)
-        except Exception:
             try:
-                return imagecodecs.jpegxl_decode(data, out=out)
+                return imagecodecs.jpegsof3_decode(data, out=out)
             except Exception:
-                raise exc
+                return imagecodecs.ljpeg_decode(data, out=out)
+        # if 'Empty JPEG image' in msg:
+        raise exc
 
 
 def jpeg_encode(
@@ -946,9 +966,7 @@ def jpeg_encode(
     smoothing=None,
     out=None,
 ):
-    """Encode JPEG 8-bit or 12-bit.
-
-    """
+    """Encode JPEG 8-bit or 12-bit."""
     if data.dtype == numpy.uint8:
         func = imagecodecs.jpeg8_encode
     elif data.dtype == numpy.uint16:
